// Pixel shader applies a one dimensional gaussian blur filter.
// This is to be used twice by the postprocess, first to
// blur horizontally, and then again to blur vertically.

sampler TextureSampler : register(s0);
float BloomScale = 2.0f;

#define SAMPLE_COUNT 15

// To get the value in C# code
float SampleCount = SAMPLE_COUNT;

float2 SampleOffsets[SAMPLE_COUNT];
float SampleWeights[SAMPLE_COUNT];


float4 PixelShader(float2 texCoord : TEXCOORD0) : COLOR0
{
    float4 c = 0;
    // Combine a number of weighted image filter taps.
    for (int i = 0; i < SAMPLE_COUNT; i++)
    {
        c += tex2D(TextureSampler, texCoord + SampleOffsets[i]) * SampleWeights[i];
    }
    return c * BloomScale;
}    

technique AdvancedGaussianBlur
{
    pass Pass1
    {
        PixelShader = compile ps_2_0 PixelShader();
        ZEnable = false;
    }
}