#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

using Xe.Graphics2D.PostProcess;
#endregion

namespace PPE
{
	/// <summary>
	/// This is the main type for your game
	/// </summary>
	public class Game1 : Microsoft.Xna.Framework.Game
	{
		GraphicsDeviceManager graphics;
		ContentManager content;

		SpriteBatch sb;
		Texture2D tex;

		PostProcessManager ppm;

		public Game1()
		{

			this.Window.AllowUserResizing = true;
			this.IsMouseVisible = true;

			// make sure there is one draw call per udpate call (else, if effect are enqueued in update, any effect can be added twice ?)
			this.IsFixedTimeStep = false;
			this.TargetElapsedTime = TimeSpan.FromMilliseconds(1);
			
			graphics = new GraphicsDeviceManager(this);
			graphics.PreferredBackBufferWidth = 1024;
			graphics.PreferredBackBufferHeight = 768;
			
			graphics.SynchronizeWithVerticalRetrace = false;

			content = new ContentManager(Services);

			
		}


		/// <summary>
		/// Allows the game to perform any initialization it needs to before starting to run.
		/// This is where it can query for any required services and load any non-graphic
		/// related content.  Calling base.Initialize will enumerate through any components
		/// and initialize them as well.
		/// </summary>
		protected override void Initialize()
		{
			// TODO: Add your initialization logic here

			ppm = new PostProcessManager(this, this.graphics.GraphicsDevice, this.content);
			this.Components.Add(ppm);

			base.Initialize();
		}


		/// <summary>
		/// Load your graphics content.  If loadAllContent is true, you should
		/// load content from both ResourceManagementMode pools.  Otherwise, just
		/// load ResourceManagementMode.Manual content.
		/// </summary>
		/// <param name="loadAllContent">Which type of content to load.</param>
		protected override void LoadGraphicsContent(bool loadAllContent)
		{
			if (loadAllContent)
			{
				// TODO: Load any ResourceManagementMode.Automatic content
				tex = (Texture2D)content.Load<Texture>(@"Content\q3art016v2");
			}

			// TODO: Load any ResourceManagementMode.Manual content
			sb = new SpriteBatch(this.graphics.GraphicsDevice);
		}


		/// <summary>
		/// Unload your graphics content.  If unloadAllContent is true, you should
		/// unload content from both ResourceManagementMode pools.  Otherwise, just
		/// unload ResourceManagementMode.Manual content.  Manual content will get
		/// Disposed by the GraphicsDevice during a Reset.
		/// </summary>
		/// <param name="unloadAllContent">Which type of content to unload.</param>
		protected override void UnloadGraphicsContent(bool unloadAllContent)
		{
			if (unloadAllContent)
			{
				// TODO: Unload any ResourceManagementMode.Automatic content
				content.Unload();
			}

			// TODO: Unload any ResourceManagementMode.Manual content
		}

		KeyboardState lastFrameKeyboardState;
		int currentEffectIndex = 0;
		/// <summary>
		/// Allows the game to run logic such as updating the world,
		/// checking for collisions, gathering input and playing audio.
		/// </summary>
		/// <param name="gameTime">Provides a snapshot of timing values.</param>
		protected override void Update(GameTime gameTime)
		{
			// Allows the game to exit
			if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed
				|| Keyboard.GetState().IsKeyDown(Keys.Escape))

				this.Exit();

			// TODO: Add your update logic here
			KeyboardState ks = Keyboard.GetState();

			// change active effect (one by one for now)
			if (!lastFrameKeyboardState.IsKeyDown(Keys.Space) && ks.IsKeyDown(Keys.Space))
			{
				if (currentEffectIndex >= ppm.EffectDictionary.Count - 1)
					currentEffectIndex = 0;
				else
					currentEffectIndex++;
			}

			lastFrameKeyboardState = ks;

			// add active effect to rendering
			this.Window.Title = ppm.EffectDictionary.Keys[currentEffectIndex] + " " + currentEffectIndex;

			ppm.AppliedEffects.Enqueue(ppm.EffectDictionary.Keys[currentEffectIndex]);

			

			base.Update(gameTime);
		}


		/// <summary>
		/// This is called when the game should draw itself.
		/// </summary>
		/// <param name="gameTime">Provides a snapshot of timing values.</param>
		protected override void Draw(GameTime gameTime)
		{
			graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

			sb.Begin();

			sb.Draw(tex, new Rectangle(0,0,1024,768), Color.White);

			sb.End();

			base.Draw(gameTime);
		}
	}
}
